{===========================================================================}
{==                                                                       ==}
{==                          VESA INFO Version 2.1                        ==}
{==                  Copyright (C) 1994-96 By The GodFather               ==}
{==                                                                       ==}
{===========================================================================}

USES Dos, GUtils;

CONST
  Version          = 'Version 2.1';
  SuperVga         = $4F;
  GetSuperVGAInfo  = $00;
  GetModeInfo      = $01;

  Models: ARRAY[0..6] OF STRING[4] =
    ('Text',                            { 0 = text }
     'CGA',                             { 1 = CGA graphics }
     'Herc',                            { 2 = Hercules graphics }
     'EGA',                             { 3 = EGA/VGA planar graphics }
     'VGA',                             { 4 = VGA packed-pixel graphics }
     'SVGA',                            { 5 = non-chain-4 256 color }
     'VESA');                           { 6-F = VESA TBD; 10-FF = OEM }

TYPE
  OEMStringType = ARRAY[1..50] OF Char;
  VideoModeType = ARRAY[1..50] OF Word;

  TVgaInfoBlock = RECORD
    VESASignature: ARRAY[1..4] OF Char;  { 0: 4 signature bytes:  must be 'VESA' }
    VESAVersion: Word;                   { 4: VESA version number }
    OEMStringPtr: ^OEMStringType;        { 6: Pointer to OEM string }
    Capabilities: LongInt;               { 10: Capabilities of the video environment }
    VideoModePtr: ^VideoModeType;        { 14: list of supported Super VGA modes }
    VIBFiller: ARRAY[1..256-18] OF Byte; { 18: fill to 256 bytes }
  END;

  TModeInfoBlock = RECORD
    ModeAttributes: Word;                { 0: mode attributes }
    WinAAttributes: Byte;                { 2: Window A attributes }
    WinBAttributes: Byte;                { 3: Window B attributes }
    WinGranularity: Word;                { 4: Window granularity }
    WinSize: Word;                       { 6: Window size }
    WinASegment: Word;                   { 8: Window A start segment }
    WinBSegment: Word;                   { 10: Window B start segment }
    WinFuncPtr: Pointer;                 { 12: (far) pointer to window function }
    BytesPerScanLine: Word;              { 14: bytes per scan line }
    XRes: Word;                          { 16: horizontal resolution }
    YRes: Word;                          { 18: vertical resolution }
    XCharSize: Byte;                     { 19: character cell width }
    YCharSize: Byte;                     { 20: character cell height }
    NumberOfPlanes: Byte;                { 21: number of memory planes }
    BitsPerPixel: Byte;                  { 22: bits per pixel }
    NumberOfBanks: Byte;                 { 23: number of banks }
    MemoryModel: Byte;                   { 24: memory model type }
    BankSize: Byte;                      { 25: bank size in KB }
    MIBfiller: ARRAY[1..256-26] OF Byte; { 26: fill to 256 bytes }
  END;

VAR
  VgaInfoBlock: TVgaInfoBlock;
  ModeInfoBlock: TModeInfoBlock;
  Regs: Registers;

PROCEDURE Error(ErrMes: STRING);
BEGIN
  Writeln(ErrMes);
  Halt(128);
END;

PROCEDURE WriteHexWord(W: Word);
CONST HexChars: ARRAY[0..$F] OF Char = '0123456789ABCDEF';
BEGIN
  Write(HexChars[Hi(W) SHR 4],
        HexChars[Hi(W) AND $F],
        HexChars[Lo(W) SHR 4],
        HexChars[Lo(W) AND $F]);
END;

FUNCTION GetOEMString: STRING;
VAR I: Byte;
BEGIN
  FOR I := 1 TO 50 DO
    IF VgaInfoBlock.OEMStringPtr^[I] = #00
      THEN Break
      ELSE GetOEMString[I] := VgaInfoBlock.OEMStringPtr^[I];
  GetOEMString[0] := Chr(I-1);
END;

PROCEDURE WriteVideoModes;
VAR
  I: Byte;
  Mode: Word;

BEGIN
  FOR I:=1 TO 50 DO
  BEGIN
    Mode := VgaInfoBlock.VideoModePtr^[I];
    IF Mode = $FFFF THEN Break;
    Regs.AH := SuperVga;
    Regs.AL := GetModeInfo;
    Regs.CX := Mode;
    Regs.ES := Seg(ModeInfoBlock);
    Regs.DI := Ofs(ModeInfoBlock);
    Intr($10, Regs);
    IF Regs.AL <> SuperVga THEN Error('Get Mode Information call is not available');
    IF Regs.AH <> 0 THEN Error('Get Mode Information call returned error');
    WITH ModeInfoBlock DO
    BEGIN
      WriteHexWord(Mode); Write('   ');
      WriteHexWord(WinASegment);
      Write('  ',XRes:5,'x ':3,YRes);
      CASE YRes OF
       1..99: Write('    ');
    100..999: Write('   ');
   999..9999: Write('  ');
      END;
      CASE BitsPerPixel OF
        4: Write(' 4 bpp');
        8: Write(' 8 bpp');
        ELSE
          Write(BitsPerPixel,' bpp');
      END;
      Write('  ',XCharSize:2,'x',YCharSize);
      IF YCharSize < 10 THEN Write(' ');
      Write('   ');
      IF MemoryModel <= 6
        THEN Write(Models[MemoryModel])
        ELSE Write('OEM N:',MemoryModel);
      Writeln;
      IF (WhereY = MaxY) AND (I=MaxY-10) THEN
      BEGIN
        FastWrite(1,WhereY,$E,PadCenter('-- Press any key to continue --',45,' '));
        Cursor(Off);
        Pause;
        Attribute(1,WhereY,$7,50);
        Cursor(On);
      END;
    END;
  END;
END;

BEGIN
  Writeln;
  Writeln('VESA INFO '+Version);
  Writeln('Copyright (C) 1995-96 By The GodFather');
  Writeln;
  Regs.AH := SuperVga;
  Regs.AL := GetSuperVGAInfo;
  Regs.ES := Seg(VgaInfoBlock);
  Regs.DI := Ofs(VgaInfoBlock);
  Intr($10, Regs);
  IF Regs.AL <> SuperVga THEN Error('VGA BIOS extensions not installed');
  IF Regs.AH <> 0 THEN Error('Get Super VGA Information call returned error');
  WITH VgaInfoBlock DO
  BEGIN
    Writeln('Super VGA BIOS extension version ',Hi(VESAVersion),'.',Lo(VESAVersion),' is installed');
    IF OEMStringPtr = NIL
      THEN Writeln('No OEM String is present')
      ELSE Writeln('OEM string is "',GetOEMString,'"');
  END;
  Writeln;
  Writeln('Mode   SSeg   Resolution     bpp    Cell   Mode');
  Writeln;
  WriteVideoModes;
END.